=== modified file 'src/key_mod/ecryptfs_key_mod_openssl.c'
--- src/key_mod/ecryptfs_key_mod_openssl.c	2009-02-09 17:29:17 +0000
+++ src/key_mod/ecryptfs_key_mod_openssl.c	2009-02-17 15:16:28 +0000
@@ -826,7 +826,7 @@
 	 .val = NULL,
 	 .display_opts = NULL,
 	 .default_val = NULL,
-	 .flags = STDIN_REQUIRED,
+	 .flags = STDIN_REQUIRED | ECRYPTFS_PARAM_FLAG_ECHO_INPUT,
 	 .num_transitions = 1,
 	 .tl = {{.val = NULL,
 		 .pretty_val = NULL,
@@ -840,7 +840,7 @@
 	 .val = NULL,
 	 .display_opts = NULL,
 	 .default_val = NULL,
-	 .flags = STDIN_REQUIRED,
+	 .flags = STDIN_REQUIRED | ECRYPTFS_PARAM_FLAG_ECHO_INPUT,
 	 .num_transitions = 1,
 	 .tl = {{.val = NULL,
 		 .pretty_val = NULL,

=== modified file 'src/key_mod/ecryptfs_key_mod_passphrase.c'
--- src/key_mod/ecryptfs_key_mod_passphrase.c	2009-02-03 08:50:36 +0000
+++ src/key_mod/ecryptfs_key_mod_passphrase.c	2009-02-17 15:16:28 +0000
@@ -33,9 +33,11 @@
 static int tf_passwd(struct ecryptfs_ctx *ctx, struct param_node *node,
 		     struct val_node **head, void **foo)
 {
+	int rc;
 	if (!node->val)
-		return MOUNT_ERROR;
-	stack_push(head, node->val);
+		return EINVAL;
+	if ((rc = stack_push(head, node->val)))
+		return -rc;
 	node->val = NULL;
 	return DEFAULT_TOK;
 }
@@ -51,7 +53,7 @@
 
 	file_head = malloc(sizeof(struct ecryptfs_name_val_pair));
 	if (!file_head) {
-		rc = -ENOMEM;
+		rc = ENOMEM;
 		goto out;
 	}
 	memset(file_head, 0, sizeof(struct ecryptfs_name_val_pair));
@@ -84,7 +86,7 @@
 		if (strcmp(walker->name, "passphrase_passwd") == 0
 		    || strcmp(walker->name, "passwd") == 0) {
 			if (asprintf(&tmp_val, "%s", walker->value) < 0) {
-				rc = -ENOMEM;
+				rc = ENOMEM;
 				goto out;
 			}
 			stack_push(head, tmp_val);
@@ -127,24 +129,24 @@
 	stack_pop_val(head, (void *)&passwd);
 	auth_tok_sig = malloc(ECRYPTFS_SIG_SIZE_HEX + 1);
 	if (!auth_tok_sig) {
-		rc = -ENOMEM;
+		rc = ENOMEM;
 		goto out;
 	}
 	from_hex(salt, salt_hex, ECRYPTFS_SIG_SIZE);
 	rc = ecryptfs_add_passphrase_key_to_keyring(auth_tok_sig, passwd, salt);
 	if (rc < 0) {
 		free(auth_tok_sig);
-		rc = MOUNT_ERROR;
+		rc = -rc;
 		goto out;
 	}
 	rc = asprintf(&param, "ecryptfs_sig=%s", auth_tok_sig);
 	if (rc == -1) {
 		free(auth_tok_sig);
-		rc = -ENOMEM;
+		rc = ENOMEM;
 		goto out;
 	}
 	free(auth_tok_sig);
-	stack_push(head, param);
+	rc = stack_push(head, param);
 out:
 	free(salt_hex);
 	free(passwd);
@@ -280,7 +282,7 @@
 
 	if (asprintf(alias, "passphrase") == -1) {
 		syslog(LOG_ERR, "Out of memory\n");
-		rc = -ENOMEM;
+		rc = ENOMEM;
 		goto out;
 	}
 out:

=== modified file 'src/libecryptfs/cmd_ln_parser.c'
--- src/libecryptfs/cmd_ln_parser.c	2009-02-03 08:50:36 +0000
+++ src/libecryptfs/cmd_ln_parser.c	2009-02-17 15:16:28 +0000
@@ -337,6 +337,10 @@
 {
 	struct ecryptfs_name_val_pair *current = head;
 	char tok_str[MAX_TOK_LEN];
+
+	if (!buf)
+		return 0;
+
 	int buf_len = strlen(buf);
 	int i, j = 0;
 	int rc = 0;

=== modified file 'src/libecryptfs/decision_graph.c'
--- src/libecryptfs/decision_graph.c	2009-02-09 17:29:17 +0000
+++ src/libecryptfs/decision_graph.c	2009-02-17 15:16:28 +0000
@@ -235,7 +235,7 @@
 				}
 				return rc;
 			}
-			else return -EINVAL;
+			else return EINVAL;
 		}
 		while (nvp) {
 			int trans_func_tok_id = NULL_TOK;
@@ -255,12 +255,12 @@
 				if ((*next = tn->next_token))
 					return 0;
 				else
-					return -EINVAL;
+					return EINVAL;
 			} else if (trans_func_tok_id == NULL_TOK) {
 				if ((*next = tn->next_token))
 					return 0;
 				else
-					return -EINVAL;
+					return EINVAL;
 			}
 			nvp = nvp->next;
 		}
@@ -275,14 +275,15 @@
 				trans_func_tok_id =
 					tn->trans_func(ctx, current,
 						       mnt_params, foo);
-			if (trans_func_tok_id == MOUNT_ERROR)
+			if (trans_func_tok_id == MOUNT_ERROR || 
+			    trans_func_tok_id > 0)
 				return trans_func_tok_id;
 			if ((*next = tn->next_token))
 				return 0;
-			else return -EINVAL;
+			else return EINVAL;
 		}
 	}
-	return MOUNT_ERROR;
+	return NULL_TOK;
 }
 
 /**
@@ -701,11 +702,10 @@
 
 	memset(*mnt_params, 0, sizeof(struct val_node));
 	rc = eval_param_tree(ctx, root_node, nvp_head, mnt_params);
-	if (rc == MOUNT_ERROR)
-		goto out;
-	else
-		rc = 0;
-out:
+	if (rc > 0)
+		return -rc;
+	if (rc != MOUNT_ERROR)
+		return 0;
 	return rc;
 }
 

=== modified file 'src/libecryptfs/module_mgr.c'
--- src/libecryptfs/module_mgr.c	2009-02-03 08:50:36 +0000
+++ src/libecryptfs/module_mgr.c	2009-02-17 15:16:28 +0000
@@ -870,7 +870,7 @@
 		}
 	}
 	ctx->nvp_head = &rc_file_nvp_head;
-	ecryptfs_eval_decision_graph(ctx, mnt_params, &root_param_node,
+	rc = ecryptfs_eval_decision_graph(ctx, mnt_params, &root_param_node,
 				     &rc_file_nvp_head);
 out_free_allowed_duplicates:
 	ad_cursor = allowed_duplicates.next;

=== modified file 'src/utils/mount.ecryptfs.c'
--- src/utils/mount.ecryptfs.c	2009-02-14 01:17:33 +0000
+++ src/utils/mount.ecryptfs.c	2009-02-17 15:16:28 +0000
@@ -100,10 +100,7 @@
 		if ((*target)[len - 1] == '/')
 			(*target)[len - 1] = '\0';
 	}
-	if (options && (argc == NUM_REQUIRED_ARGS)) {
-		fprintf(stderr, "Not enough remaining data for options\n");
-		return -EINVAL;
-	} else if ((options) && (argc > NUM_REQUIRED_ARGS)) {
+	if ((options) && (argc >= NUM_REQUIRED_ARGS)) {
 		int i;
 
 		ptr = NULL;
@@ -116,8 +113,7 @@
 			fprintf(stderr, "Unable to find a list of options to "
 					"parse, defaulting to interactive "
 					"mount\n");
-			rc = -ENOENT;
-			goto out;
+			return 0;
 		}
 		len = strlen(ptr) + 1; /* + NULL-terminator */
 		*options = malloc(len);
@@ -147,6 +143,9 @@
 	char *end;
 	int num_options = 0;
 
+	if (!options)
+		return 0;
+
 	end = strchr(options, '\0');
 	opt = options;
 	while (opt) {
@@ -255,6 +254,9 @@
 	size_t len;
 	int used = 0, first = 1;
 
+	if (!options)
+		return 0;
+
 	len = (strlen(options) + 1);
 	if ((temp = (char*)malloc(len)) == NULL) {
 		fprintf(stderr, "Out of memory\n");
@@ -292,7 +294,6 @@
 static int process_sig(char *auth_tok_sig, struct passwd *pw)
 {
 	char *home;
-	uid_t id;
 	char *sig_cache_filename = NULL;
 	char *dot_ecryptfs_dir;
 	int flags;
@@ -362,7 +363,7 @@
 				       "file; continuing with mount.\n");
 		} else {
 			printf("Aborting mount.\n");
-			rc = -EINVAL;
+			rc = ECANCELED;
 			goto out;
 		}
 	}
@@ -376,6 +377,10 @@
 	char *opt;
 	char *next_opt;
 	char *end;
+
+	if (!options || !option)
+		return 0;
+
 	int option_len = strlen(option);
 
 	end = strchr(options, '\0');
@@ -418,7 +423,7 @@
 		if (!opts_str_contains_option(opts, required_mount_opts[i])) {
 			printf("Required mount option not provided: [%s]\n",
 			       required_mount_opts[i]);
-			rc = 1;
+			rc = -EINVAL;
 			goto out;
 		}
 		i++;
@@ -454,12 +459,15 @@
 	if (rc)
 		goto out;
 	num_opts = ecryptfs_generate_mount_flags(opts, &flags);
-	if (asprintf(&new_opts, "%secryptfs_unlink_sigs,", opts) == -1) {
-		new_opts = NULL;
+	if (!(temp = strdup("ecryptfs_unlink_sigs"))) {
 		rc = -ENOMEM;
 		goto out;
 	}
+	if ((rc = stack_push(&mnt_params, (void *)temp)))
+		goto out;
 	printf("Attempting to mount with the following options:\n");
+	new_opts = opts;
+	opts = NULL;
 	while (!stack_pop_val(&mnt_params, (void *)&val)) {
 		if(!val)
 			break;
@@ -467,12 +475,15 @@
 		printf("  %s\n", val);
 		if (sig_cache && memcmp(val, "ecryptfs_sig=", 13) == 0) {
 			if ((rc = process_sig(&val[13], pw))) {
-				printf("Error processing sig; rc = [%d]\n", rc);
+				if (rc != ECANCELED)
+					printf("Error processing sig; "
+					       "rc = [%d]\n", rc);
 				goto out;
 			}
 		}
-		if (!strstr(temp, val)) {
-			rc = asprintf(&new_opts, "%s,%s", val, temp);
+		if (!temp || !strstr(temp, val)) {
+			rc = asprintf(&new_opts, "%s%c%s", val,
+				      ((temp && *temp) ? ',' : '\0'), temp);
 			if (rc == -1) {
 				new_opts = NULL;
 				rc = -ENOMEM;
@@ -521,8 +532,8 @@
 
 	rc = mlockall(MCL_FUTURE);
 	if (rc) {
-		fprintf(stderr, "Exiting. Unable to mlockall address space\n");
-		return 0;
+		fprintf(stderr, "Exiting. Unable to mlockall address space: %m\n");
+		return -1;
 	}
 	if (dump_args) {
 		int i;
@@ -588,22 +599,24 @@
 			&ctx, &mnt_params, version, opts_str,
 			ECRYPTFS_ASK_FOR_ALL_MOUNT_OPTIONS);
 		if (rc) {
-			printf("Error attempting to evaluate mount options; "
-			       "rc = [%d]. See your system logs for more "
-			       "details on why this happened. Try "
-			       "updating/reinstalling your "
-			       "ecryptfs-utils package, contact your operating "
-			       "system vendor, and/or submit a bug "
-			       "report on the ecryptfs-devel mailing list on "
-			       "Launchpad.\n", rc);
+			printf("Error attempting to evaluate mount options: "
+			       "[%d] %s\nCheck your system logs for details "
+			       "on why this happened.\nTry updating your "
+			       "ecryptfs-utils package, and/or\nsubmit a bug "
+			       "report on https://launchpad.net/ecryptfs\n",
+				rc, strerror(-rc));
 			goto out;
 		}
 		rc = ecryptfs_do_mount(argc, argv, mnt_params, sig_cache, pw);
+		if (rc == ECANCELED) {
+		    rc = 0;
+		    goto out;
+		}
 		if (rc) {
 			if (rc > 0)
 				rc = -rc;
-			printf("Error mounting eCryptfs; rc = [%d]; strerr = "
-			       "[%s]. Check your system logs; visit "
+			printf("Error mounting eCryptfs: [%d] %s\n"
+			       "Check your system logs; visit "
 			       "<http://launchpad.net/ecryptfs>\n",
 			       rc, strerror(-rc));
 			if (rc == -ENODEV)

